"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LoaderBuilder = void 0;
const plugin_compiler_alipay_1 = require("@morjs/plugin-compiler-alipay");
const utils_1 = require("@morjs/utils");
const path_1 = __importDefault(require("path"));
const constants_1 = require("../constants");
class LoaderBuilder {
    constructor(loader) {
        var _a;
        this.loader = loader;
        this.loaderOptions = loader.getOptions();
        this.userConfig = this.loaderOptions.userConfig;
        this.isAtomicMode = ((_a = this.userConfig) === null || _a === void 0 ? void 0 : _a.compileMode) !== 'bundle';
        this.entryBuilder = this.loaderOptions.entryBuilder;
        this.callback = loader.async();
        this.getRelevants(loader.resourcePath);
    }
    static process(loader, fn) {
        return new LoaderBuilder(loader).compile(fn);
    }
    getResourceName() {
        const name = path_1.default.parse(this.loader.resourcePath).name;
        const conditionalFileExts = this.loaderOptions.conditionalCompileFileExt ||
            (0, utils_1.asArray)(constants_1.defaultConditionalFileExt);
        if (!conditionalFileExts.length)
            return name;
        return name.replace(new RegExp(`\\.(${conditionalFileExts.map((e) => e.slice(1)).join('|')})$`), '');
    }
    // 创建当前文件的编译选项
    async generateOptions() {
        const opts = this.loaderOptions;
        const appConfig = (await this.loadAppConfig()) || {};
        appConfig.usingComponents = {
            ...(opts.usingComponents || {}),
            ...(appConfig.usingComponents || {})
        };
        let appStyleFilePath = this.entryBuilder.globalStyleFilePath;
        // 因为 tiga 编译逻辑会针对 / 开头的路径追加 @
        // 这里转换为相对路径
        if (appStyleFilePath) {
            appStyleFilePath = path_1.default.basename(appStyleFilePath);
        }
        const appConfigPath = this.getAppConfigFilePath();
        return {
            ...opts,
            isAtomicMode: this.isAtomicMode,
            name: this.getResourceName(),
            config: await this.loadRelatedConfig(),
            appConfig,
            hasAppConfig: !!appConfigPath,
            appConfigPath,
            resourcePath: this.loader.resourcePath,
            rootPath: this.userConfig.srcPath,
            globalComponentsConfig: opts.globalComponentsConfig,
            rpxRootValue: opts.rpxRootValue || 32,
            platform: opts.platform || 'h5',
            conditionalCompileFileExt: opts.conditionalCompileFileExt,
            appStyleFilePath,
            ...this.relevantPaths
        };
    }
    getRelevants(resourcePath) {
        var _a, _b, _c, _d;
        const eb = this.entryBuilder;
        const entry = eb.getEntryByFilePath(resourcePath);
        if (!entry) {
            this.relevantEntries = {};
            this.relevantPaths = {};
            return;
        }
        // 组件编译模式下使用 fullEntryName
        const field = this.isAtomicMode ? 'fullEntryName' : 'fullPath';
        const contextPath = path_1.default.dirname(entry[field]);
        this.relevantEntries = {
            scriptEntry: eb.entries.get(entry.entryName),
            templateEntry: eb.entries.get(`${entry.entryName}${plugin_compiler_alipay_1.fileType.template}${this.isAtomicMode ? '.js' : ''}`),
            styleEntry: eb.entries.get(`${entry.entryName}${this.isAtomicMode ? '.css' : plugin_compiler_alipay_1.fileType.style}`),
            configEntry: eb.entries.get(`${entry.entryName}${plugin_compiler_alipay_1.fileType.config}`)
        };
        this.relevantPaths = {
            scriptFilePath: this.getRelativePath(contextPath, (_a = this.relevantEntries.scriptEntry) === null || _a === void 0 ? void 0 : _a[field]),
            templateFilePath: this.getRelativePath(contextPath, (_b = this.relevantEntries.templateEntry) === null || _b === void 0 ? void 0 : _b[field]),
            styleFilePath: this.getRelativePath(contextPath, (_c = this.relevantEntries.styleEntry) === null || _c === void 0 ? void 0 : _c[field]),
            configFilePath: this.getRelativePath(contextPath, (_d = this.relevantEntries.configEntry) === null || _d === void 0 ? void 0 : _d[field])
        };
    }
    getRelativePath(contextPath, targetPath) {
        if (!targetPath)
            return;
        const relativePath = (0, utils_1.slash)(path_1.default.relative(contextPath, targetPath));
        if (relativePath.startsWith('.'))
            return relativePath;
        return `./${relativePath}`;
    }
    async loadModule(request) {
        return await new Promise((resolve, reject) => {
            this.loader.loadModule(request, (err, source, sourceMap, module) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve({ source, sourceMap, module });
                }
            });
        });
    }
    async compile(fn) {
        const options = await this.generateOptions();
        try {
            const result = await fn(options);
            this.callback(null, result);
        }
        catch (err) {
            this.callback(err);
        }
    }
    // 获取当前文件的config
    async loadRelatedConfig() {
        var _a;
        const configFilePath = (_a = this.relevantEntries.configEntry) === null || _a === void 0 ? void 0 : _a.fullPath;
        if (configFilePath) {
            const { module } = (await this.loadModule(configFilePath));
            return this.getModuleJsonData(module);
        }
        else {
            return null;
        }
    }
    getModuleJsonData(module) {
        if (!(module && module.buildInfo && module.buildInfo.jsonData)) {
            return {};
        }
        const { jsonData } = module.buildInfo;
        return typeof jsonData.get === 'function' ? jsonData.get() : jsonData;
    }
    getAppConfigFilePath() {
        if (this.userConfig.compileType === 'miniprogram') {
            return this.entryBuilder.globalConfigFilePath;
        }
    }
    // 获取 app.json 配置
    async loadAppConfig() {
        const appJsonPath = this.getAppConfigFilePath();
        if (appJsonPath) {
            const { module } = await this.loadModule(appJsonPath);
            return this.getModuleJsonData(module);
        }
        return {};
    }
}
exports.LoaderBuilder = LoaderBuilder;
//# sourceMappingURL=builder.js.map